@@ -1,61 +1,79 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from decimal import Decimal
+from functools import reduce
 from trytond.model import ModelView, ModelSQL, fields, OPERATORS
 from trytond.backend import TableHandler
 from trytond.pyson import In, Eval, Not, Equal, If, Get, Bool
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 STATES = {
     'readonly': In(Eval('state'), ['cancel', 'assigned', 'done']),
 }
+DEPENDS = ['state']
 
 
 class Move(ModelSQL, ModelView):
     "Stock Move"
     _name = 'stock.move'
     _description = __doc__
+    _order_name = 'product'
     product = fields.Many2One("product.product", "Product", required=True,
-            select=1, states=STATES,
-            on_change=['product', 'currency', 'uom', 'company',
-                'from_location', 'to_location'],
-            domain=[('type', '!=', 'service')])
+        select=1, states=STATES,
+        on_change=['product', 'currency', 'uom', 'company',
+            'from_location', 'to_location'],
+        domain=[('type', '!=', 'service')],
+        depends=DEPENDS)
     uom = fields.Many2One("product.uom", "Uom", required=True, states=STATES,
-            domain=[
-                ('category', '=',
-                    (Eval('product'), 'product.default_uom.category')),
+        domain=[
+            ('category', '=',
+                (Eval('product'), 'product.default_uom.category')),
             ],
-            context={
-                'category': (Eval('product'), 'product.default_uom.category'),
+        context={
+            'category': (Eval('product'), 'product.default_uom.category'),
             },
-            on_change=['product', 'currency', 'uom', 'company',
-                'from_location', 'to_location'])
+        on_change=['product', 'currency', 'uom', 'company',
+            'from_location', 'to_location'],
+        depends=['state', 'product'])
     unit_digits = fields.Function(fields.Integer('Unit Digits',
         on_change_with=['uom']), 'get_unit_digits')
     quantity = fields.Float("Quantity", required=True,
-            digits=(16, Eval('unit_digits', 2)), states=STATES)
+        digits=(16, Eval('unit_digits', 2)), states=STATES,
+        depends=['state', 'unit_digits'])
     internal_quantity = fields.Float('Internal Quantity', readonly=True,
         required=True)
     from_location = fields.Many2One("stock.location", "From Location", select=1,
-            required=True, states=STATES,
-            domain=[('type', 'not in', ('warehouse', 'view'))])
+        required=True, states=STATES, depends=DEPENDS,
+        domain=[('type', 'not in', ('warehouse', 'view'))])
     to_location = fields.Many2One("stock.location", "To Location", select=1,
-            required=True, states=STATES,
-            domain=[('type', 'not in', ('warehouse', 'view'))])
+        required=True, states=STATES, depends=DEPENDS,
+        domain=[('type', 'not in', ('warehouse', 'view'))])
     shipment_in = fields.Many2One('stock.shipment.in', 'Supplier Shipment',
-            readonly=True, select=1, ondelete='CASCADE')
+        domain=[('company', '=', Eval('company'))], depends=['company'],
+        readonly=True, select=1, ondelete='CASCADE')
     shipment_out = fields.Many2One('stock.shipment.out', 'Customer Shipment',
-            readonly=True, select=1, ondelete='CASCADE')
+        domain=[('company', '=', Eval('company'))], depends=['company'],
+        readonly=True, select=1, ondelete='CASCADE')
     shipment_out_return = fields.Many2One('stock.shipment.out.return',
-            'Customer Return Shipment', readonly=True, select=1,
-            ondelete='CASCADE')
+        'Customer Return Shipment', readonly=True, select=1,
+        domain=[('company', '=', Eval('company'))], depends=['company'],
+        ondelete='CASCADE')
     shipment_in_return = fields.Many2One('stock.shipment.in.return',
-            'Supplier Return Shipment', readonly=True, select=1,
-            ondelete='CASCADE')
+        'Supplier Return Shipment', readonly=True, select=1,
+        domain=[('company', '=', Eval('company'))], depends=['company'],
+        ondelete='CASCADE')
     shipment_internal = fields.Many2One('stock.shipment.internal',
-            'Internal Shipment', readonly=True, select=1, ondelete='CASCADE')
-    planned_date = fields.Date("Planned Date", states=STATES, select=2)
+        'Internal Shipment', readonly=True, select=1, ondelete='CASCADE',
+        domain=[('company', '=', Eval('company'))], depends=['company'])
+    planned_date = fields.Date("Planned Date", states={
+            'readonly': (In(Eval('state'), ['cancel', 'assigned', 'done'])
+                | Eval('shipment_in') | Eval('shipment_out')
+                | Eval('shipment_in_return') | Eval('shipment_out_return')
+                | Eval('shipment_internal'))
+            }, depends=['state', 'shipment_in', 'shipment_out',
+            'shipment_in_return', 'shipment_out_return', 'shipment_internal'],
+        select=2)
     effective_date = fields.Date("Effective Date", readonly=True, select=2)
     state = fields.Selection([
         ('draft', 'Draft'),
@@ -64,25 +82,29 @@ class Move(ModelSQL, ModelView):
         ('cancel', 'Canceled'),
         ], 'State', select=1, readonly=True)
     company = fields.Many2One('company.company', 'Company', required=True,
-            states={
-                'readonly': Not(Equal(Eval('state'), 'draft')),
-            }, domain=[
-                ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
-            ])
+        states={
+            'readonly': Not(Equal(Eval('state'), 'draft')),
+            },
+        domain=[
+            ('id', If(In('company', Eval('context', {})), '=', '!='),
+                Get(Eval('context', {}), 'company', 0)),
+            ],
+        depends=['state'])
     unit_price = fields.Numeric('Unit Price', digits=(16, 4),
-            states={
-                'invisible': Not(Bool(Eval('unit_price_required'))),
-                'required': Bool(Eval('unit_price_required')),
-                'readonly': Not(Equal(Eval('state'), 'draft')),
-            })
+        states={
+            'invisible': Not(Bool(Eval('unit_price_required'))),
+            'required': Bool(Eval('unit_price_required')),
+            'readonly': Not(Equal(Eval('state'), 'draft')),
+            },
+        depends=['unit_price_required', 'state'])
     cost_price = fields.Numeric('Cost Price', digits=(16, 4), readonly=True)
     currency = fields.Many2One('currency.currency', 'Currency',
-            states={
-                'invisible': Not(Bool(Eval('unit_price_required'))),
-                'required': Bool(Eval('unit_price_required')),
-                'readonly': Not(Equal(Eval('state'), 'draft')),
-            })
+        states={
+            'invisible': Not(Bool(Eval('unit_price_required'))),
+            'required': Bool(Eval('unit_price_required')),
+            'readonly': Not(Equal(Eval('state'), 'draft')),
+            },
+        depends=['unit_price_required', 'state'])
     unit_price_required = fields.Function(fields.Boolean('Unit Price Required',
         on_change_with=['from_location', 'to_location']),
         'get_unit_price_required')
@@ -173,8 +195,8 @@ class Move(ModelSQL, ModelView):
         return Transaction().context.get('planned_date') or False
 
     def default_to_location(self):
-        location_obj = self.pool.get('stock.location')
-        party_obj = self.pool.get('party.party')
+        location_obj = Pool().get('stock.location')
+        party_obj = Pool().get('party.party')
         res = False
 
         warehouse = None
@@ -200,8 +222,8 @@ class Move(ModelSQL, ModelView):
         return res
 
     def default_from_location(self):
-        location_obj = self.pool.get('stock.location')
-        party_obj = self.pool.get('party.party')
+        location_obj = Pool().get('stock.location')
+        party_obj = Pool().get('party.party')
         res = False
 
         warehouse = None
@@ -236,8 +258,8 @@ class Move(ModelSQL, ModelView):
         return Transaction().context.get('company') or False
 
     def default_currency(self):
-        company_obj = self.pool.get('company.company')
-        currency_obj = self.pool.get('currency.currency')
+        company_obj = Pool().get('company.company')
+        currency_obj = Pool().get('currency.currency')
         company = Transaction().context.get('company')
         if company:
             company = company_obj.browse(company)
@@ -245,7 +267,7 @@ class Move(ModelSQL, ModelView):
         return False
 
     def on_change_with_unit_digits(self, vals):
-        uom_obj = self.pool.get('product.uom')
+        uom_obj = Pool().get('product.uom')
         if vals.get('uom'):
             uom = uom_obj.browse(vals['uom'])
             return uom.digits
@@ -258,11 +280,12 @@ class Move(ModelSQL, ModelView):
         return res
 
     def on_change_product(self, vals):
-        product_obj = self.pool.get('product.product')
-        uom_obj = self.pool.get('product.uom')
-        currency_obj = self.pool.get('currency.currency')
-        company_obj = self.pool.get('company.company')
-        location_obj = self.pool.get('stock.location')
+        pool = Pool()
+        product_obj = pool.get('product.product')
+        uom_obj = pool.get('product.uom')
+        currency_obj = pool.get('currency.currency')
+        company_obj = pool.get('company.company')
+        location_obj = pool.get('stock.location')
 
         res = {
             'unit_price': Decimal('0.0'),
@@ -290,11 +313,12 @@ class Move(ModelSQL, ModelView):
         return res
 
     def on_change_uom(self, vals):
-        product_obj = self.pool.get('product.product')
-        uom_obj = self.pool.get('product.uom')
-        currency_obj = self.pool.get('currency.currency')
-        company_obj = self.pool.get('company.company')
-        location_obj = self.pool.get('stock.location')
+        pool = Pool()
+        product_obj = pool.get('product.product')
+        uom_obj = pool.get('product.uom')
+        currency_obj = pool.get('currency.currency')
+        company_obj = pool.get('company.company')
+        location_obj = pool.get('stock.location')
 
         res = {
             'unit_price': Decimal('0.0'),
@@ -328,7 +352,7 @@ class Move(ModelSQL, ModelView):
         return self.on_change_with_unit_price_required(vals)
 
     def on_change_with_unit_price_required(self, vals):
-        location_obj = self.pool.get('stock.location')
+        location_obj = Pool().get('stock.location')
         if vals.get('from_location'):
             from_location = location_obj.browse(vals['from_location'])
             if from_location.type == 'supplier':
@@ -356,7 +380,7 @@ class Move(ModelSQL, ModelView):
         return True
 
     def check_period_closed(self, ids):
-        period_obj = self.pool.get('stock.period')
+        period_obj = Pool().get('stock.period')
         period_ids = period_obj.search([
             ('state', '=', 'closed'),
         ], order=[('date', 'DESC')], limit=1)
@@ -383,7 +407,7 @@ class Move(ModelSQL, ModelView):
 
     def search(self, args, offset=0, limit=None, order=None, count=False,
             query_string=False):
-        location_obj = self.pool.get('stock.location')
+        location_obj = Pool().get('stock.location')
 
         args = args[:]
         def process_args(args):
@@ -421,13 +445,14 @@ class Move(ModelSQL, ModelView):
         :param company: the company id ot a BrowseRecord of the company
         :param date: the date for the currency rate calculation
         """
-        uom_obj = self.pool.get('product.uom')
-        product_obj = self.pool.get('product.product')
-        product_template_obj = self.pool.get('product.template')
-        location_obj = self.pool.get('stock.location')
-        currency_obj = self.pool.get('currency.currency')
-        company_obj = self.pool.get('company.company')
-        date_obj = self.pool.get('ir.date')
+        pool = Pool()
+        uom_obj = pool.get('product.uom')
+        product_obj = pool.get('product.product')
+        product_template_obj = pool.get('product.template')
+        location_obj = pool.get('stock.location')
+        currency_obj = pool.get('currency.currency')
+        company_obj = pool.get('company.company')
+        date_obj = pool.get('ir.date')
 
         if isinstance(uom, (int, long)):
             uom = uom_obj.browse(uom)
@@ -474,17 +499,18 @@ class Move(ModelSQL, ModelView):
                 })
 
     def _get_internal_quantity(self, quantity, uom, product):
-        uom_obj = self.pool.get('product.uom')
+        uom_obj = Pool().get('product.uom')
         internal_quantity = uom_obj.compute_qty(uom, quantity,
                 product.default_uom, round=True)
         return internal_quantity
 
 
     def create(self, vals):
-        location_obj = self.pool.get('stock.location')
-        product_obj = self.pool.get('product.product')
-        uom_obj = self.pool.get('product.uom')
-        date_obj = self.pool.get('ir.date')
+        pool = Pool()
+        location_obj = pool.get('stock.location')
+        product_obj = pool.get('product.product')
+        uom_obj = pool.get('product.uom')
+        date_obj = pool.get('ir.date')
 
         today = date_obj.today()
         vals = vals.copy()
@@ -522,7 +548,7 @@ class Move(ModelSQL, ModelView):
         return super(Move, self).create(vals)
 
     def write(self, ids, vals):
-        date_obj = self.pool.get('ir.date')
+        date_obj = Pool().get('ir.date')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -646,10 +672,11 @@ class Move(ModelSQL, ModelView):
         :param moves: a BrowseRecordList of stock.move to assign
         :return: True if succeed or False if not
         '''
-        product_obj = self.pool.get('product.product')
-        uom_obj = self.pool.get('product.uom')
-        date_obj = self.pool.get('ir.date')
-        location_obj = self.pool.get('stock.location')
+        pool = Pool()
+        product_obj = pool.get('product.product')
+        uom_obj = pool.get('product.uom')
+        date_obj = pool.get('ir.date')
+        location_obj = pool.get('stock.location')
 
         Transaction().cursor.lock(self._table)
 
